package com.nh.farm.cloud.util.redis;

import com.nh.farm.cloud.basic.entity.LoginUser;
import com.nh.farm.cloud.basic.entity.UserEntity;
import com.nh.farm.cloud.util.ServletUtils;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.concurrent.TimeUnit;

/**
 * token验证处理
 */
@Component
public class TokenUtil {
    @Autowired
    private RedisCacheUtil redisCacheUtil;

    protected static final long MILLIS_SECOND = 1000;

    protected static final long MILLIS_MINUTE = 60 * MILLIS_SECOND;

    private final static long expireTime = CacheConstants.EXPIRATION;

    private final static String ACCESS_TOKEN = CacheConstants.LOGIN_TOKEN_KEY;

    private final static Long MILLIS_MINUTE_TEN = CacheConstants.REFRESH_TIME * MILLIS_MINUTE;

    /**
     * 获取用户身份信息
     *
     * @return 用户信息
     */
    public LoginUser getLoginUser(HttpServletRequest request) {
        // 获取请求携带的令牌
        String token = getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            Claims claims = parseToken(token);
            // 解析对应的权限以及用户信息
            String uuid = (String) claims.get(ACCESS_TOKEN);
            String userKey = getTokenKey(uuid);
            return redisCacheUtil.getCacheObject(userKey);
        }
        return null;
    }

    /**
     * 设置用户身份信息
     */
    public void setLoginUser(LoginUser loginUser) {
        if (loginUser != null && StringUtils.isNotEmpty(loginUser.getToken())) {
            refreshToken(loginUser);
        }
    }



    /**
     * 验证令牌有效期，相差不足20分钟，自动刷新缓存
     */
    public void verifyToken(LoginUser loginUser) {
        long expireTime = loginUser.getExpireTime();
        long currentTime = System.currentTimeMillis();
        if (expireTime - currentTime <= MILLIS_MINUTE_TEN) {
            refreshToken(loginUser);
        }
    }

    /**
     * 刷新令牌有效期
     *
     * @param loginUser 登录信息
     */
    public void refreshToken(LoginUser loginUser) {
        loginUser.setLoginTime(System.currentTimeMillis());
        loginUser.setExpireTime(loginUser.getLoginTime() + expireTime * MILLIS_MINUTE);
        // 根据uuid将loginUser缓存
        String userKey = getTokenKey(loginUser.getToken());
        redisCacheUtil.setCacheObject(userKey, loginUser, expireTime, TimeUnit.MINUTES);
    }

    /**
     * 从令牌中获取数据声明
     *
     * @param token 令牌
     * @return 数据声明
     */
    private Claims parseToken(String token) {
        return Jwts.parser()
                .setSigningKey("nhtccmcrocommonbasicsecret")
                .parseClaimsJws(token)
                .getBody();
    }


    /**
     * 从令牌中获取用户名
     *
     * @param token 令牌
     * @return 用户名
     */
    public String getUsernameFromToken(String token) {
        Claims claims = parseToken(token);
        return claims.getSubject();
    }

    /**
     * 获取请求token 加密
     *
     * @return token
     */
    public String getToken(HttpServletRequest request) {
        String token = request.getHeader("authorization");

        if (StringUtils.isNotEmpty(token) && token.startsWith("Bearer ")) {
            token = token.replace("Bearer ", "");
            return token;
        }
        return null;
    }

    private String getTokenKey(String uuid) {
        return ACCESS_TOKEN + uuid;
    }



    /**
     * 查询当前登陆人信息
     *
     * @return true 是
     */
    public UserEntity getUser() {

        return this.getLoginUser(ServletUtils.getRequest()).getSysUser();
    }
    /**
     * 查询当前登陆人是否为管理员
     *
     * @return true 是
     */
    public Boolean getIsAdmin() {

        return this.getLoginUser(ServletUtils.getRequest()).getSysUser().getIsAdmin();
    }

    /**
     * 查询当前登陆人的用户id
     */
    public String getUsername() {

        return this.getLoginUser(ServletUtils.getRequest()).getSysUser().getUsername();
    }

    /**
     * 查询当前登陆人的名称
     */
    public String getNickName() {

        return this.getLoginUser(ServletUtils.getRequest()).getSysUser().getNickName();
    }

    /**
     * 查询当前登陆人的名称
     */
    public String getBankCode() {

        return this.getLoginUser(ServletUtils.getRequest()).getSysUser().getBankCode();
    }

    /**
     * 查询当前登陆人的名称
     */
    public String getKid() {
        String kid = "519848949849";
        try {
            kid = this.getLoginUser(ServletUtils.getRequest()).getSysUser().getKid();
        } catch (Exception e) {

        }
        return kid;
    }

    public Integer getBankLevel() {
        return this.getLoginUser(ServletUtils.getRequest()).getBankLevel();
    }
}
